local Addon, private = ...

setfenv(1, private)

-- Private methods
-- ============================================================================

local window
local navigator
local context
local sidebar
local modules
local bookmarks
local pageModule
local pagePath
local pageTitle

local function isInsideResizeButton(self)
	local mouse = Inspect.Mouse()
	return mouse.x - self:GetLeft() >= self:GetBottom() - mouse.y
end

local function createResizeButton(self, minWidth, minHeight, callback)
	local btn = UI.CreateFrame("Texture", "", self:GetContent())
	
	btn:SetTexture("Rift", "chat_resize_(normal).png.dds")
	btn:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 8, 8)
	btn.start = nil
	btn.width = nil
	btn.height = nil
	btn:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self)
		if(isInsideResizeButton(self)) then
			self:SetTexture("Rift", self.start and "chat_resize_(click).png.dds" or "chat_resize_(over).png.dds")
		end
	end, "")
	btn:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self)
		self:SetTexture("Rift", self.start and "chat_resize_(click).png.dds" or "chat_resize_(normal).png.dds")
	end, "")
	btn:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self) self:SetTexture("Rift", "chat_resize_(over).png.dds")
		self.start = nil
	end, "")
	btn:EventAttach(Event.UI.Input.Mouse.Left.Upoutside, function(self)
		self:SetTexture("Rift", "chat_resize_(normal).png.dds")
		self.start = nil
	end, "")
	btn:EventAttach(Event.UI.Input.Mouse.Left.Down, function()
		if(isInsideResizeButton(btn)) then
			btn:SetTexture("Rift", "chat_resize_(click).png.dds")
			btn.start = Inspect.Mouse()
			btn.width = self:GetWidth()
			btn.height = self:GetHeight()
		end
	end, "")
	btn:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function()
		if(btn.start) then
			local mouse = Inspect.Mouse()
			callback(math.max(minWidth, btn.width + mouse.x - btn.start.x), math.max(minHeight, btn.height + mouse.y - btn.start.y))
		elseif(isInsideResizeButton(btn)) then
			btn:SetTexture("Rift", "chat_resize_(over).png.dds")
		else
			btn:SetTexture("Rift", "chat_resize_(normal).png.dds")
		end
	end, "")
end

local function makeDraggable(self)
	local dx = nil
	local dy = nil
	local function leftDown()
		local mouse = Inspect.Mouse()
		dx = mouse.x - self:GetLeft()
		dy = mouse.y - self:GetTop()
	end
	local function leftUp()
		dx = nil
		dy = nil
	end
	local function mouseMove()
		if(dx) then
			local mouse = Inspect.Mouse()
			self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", mouse.x - dx, mouse.y - dy)
		end
	end
	
	local content = self:GetContent()
	local border = self:GetBorder()
	content:EventAttach(Event.UI.Input.Mouse.Left.Down, leftDown, "")
	content:EventAttach(Event.UI.Input.Mouse.Left.Up, leftUp, "")
	content:EventAttach(Event.UI.Input.Mouse.Left.Upoutside, leftUp, "")
	content:EventAttach(Event.UI.Input.Mouse.Cursor.Move, mouseMove, "")

	border:EventAttach(Event.UI.Input.Mouse.Left.Down, leftDown, "")
	border:EventAttach(Event.UI.Input.Mouse.Left.Up, leftUp, "")
	border:EventAttach(Event.UI.Input.Mouse.Left.Upoutside, leftUp, "")
	border:EventAttach(Event.UI.Input.Mouse.Cursor.Move, mouseMove, "")
end

local function createSidebar(parent)
	local self = UI.CreateFrame("Frame", "", parent)
	self:SetBackgroundColor(0, 0, 0, 0.5)

	local rift = UI.CreateFrame("Text", "", self)
	rift:SetPoint("TOPCENTER", self, "TOPCENTER", 0, Const.padding)
	rift:SetFontSize(16)
	
	rift:SetText(string.format(Const.moduleLinkFormat, "Rift", Const.indexPath, "Rift"), true)
	
	local sep = UI.CreateFrame("Texture", "", self)
	sep:SetPoint("TOP", rift, "BOTTOM", nil, -10)
	sep:SetPoint("RIGHT", self, "RIGHT")
	sep:SetPoint("LEFT", self, "LEFT")
	sep:SetTexture("Rift", "divider_01a.png.dds")
	
	local btn = UI.CreateFrame("Texture", "", self)
	btn:SetTexture("Rift", "icon_menu_achievements_(disabled).png.dds")
	btn:SetPoint("TOPLEFT", self, "TOPLEFT")
	btn:EventAttach(Event.UI.Input.Mouse.Left.Click, function()
		if(modules:GetVisible() and not modules:FadingOut()) then
			modules:FadeOut()
			bookmarks:FadeIn()
			btn:SetTexture("Rift", "icon_menu_achievements.png.dds")
		else
			modules:FadeIn()
			bookmarks:FadeOut()
			btn:SetTexture("Rift", "icon_menu_achievements_(disabled).png.dds")
		end
	end, "")
	
	self:SetHeight(sep:GetBottom() - sep:GetHeight() / 2 - rift:GetTop())
	
	return self
end

local function createResizeGripper(parent)
	local self = UI.CreateFrame("Texture", "", parent)
	self:SetWidth(10)
	self:SetAlpha(0.5)
	self:SetTexture("Rift", "indicator_tracking.png.dds")
	
	local width
	local x
	local function down()
		x = Inspect.Mouse().x
		width = sidebar:GetWidth()
	end
	local function up()
		x = nil
		width = nil
	end
	local function move()
		if(width) then
			sidebar:SetWidth(math.max(80, width + Inspect.Mouse().x - x))
		end
	end
	self:EventAttach(Event.UI.Input.Mouse.Left.Down, down, "")
	self:EventAttach(Event.UI.Input.Mouse.Left.Up, up, "")
	self:EventAttach(Event.UI.Input.Mouse.Left.Upoutside, up, "")
	self:EventAttach(Event.UI.Input.Mouse.Cursor.Move, move, "")
	
	return self
end

window = function()
	context = UI.CreateContext(Addon.identifier)
	local self = UI.CreateFrame("RiftWindow", "Main", context)
	self:SetVisible(false)
	self:SetTitle(Addon.toc.Name)
	self:SetController("content")
	local settings = _G.ApiBrowserWindowSettings or { }
	settings = settings.mainWindow
	if(settings and settings) then
		self:SetWidth(settings.width)
		self:SetHeight(settings.height)
		self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", settings.x, settings.y)
	else
		self:SetWidth(800)
		self:SetHeight(600)
		self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (UIParent:GetWidth() - self:GetWidth()) / 2, (UIParent:GetHeight() - self:GetHeight()) / 2)
	end
	window = self
	
	makeDraggable(self)
	Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, function(h, identifier)
		if(identifier == Addon.identifier) then
			_G.ApiBrowserWindowSettings = _G.ApiBrowserWindowSettings or { }
			_G.ApiBrowserWindowSettings.mainWindow = {
				x = self:GetLeft(),
				y = self:GetTop(),
				width = self:GetWidth(),
				height = self:GetHeight(),
				sidebar = {
					width = sidebar:GetWidth(),
				},
			}
		end
	end, "")
	
	local content = self:GetContent()
	local border = self:GetBorder()
	
	local close = UI.CreateFrame("RiftButton", "", self)
	close:SetSkin("close")
	close:SetPoint("TOPRIGHT", border, "TOPRIGHT", -8, 16)
	close:EventAttach(Event.UI.Button.Left.Press, function() self:FadeOut() end, "")
	
	createResizeButton(self, 400, 400, function(width, height)
		self:SetWidth(width)
		self:SetHeight(height)
	end)
	
	sidebar = createSidebar(self)
	sidebar:SetPoint("TOPLEFT", content, "TOPLEFT", Const.padding, Const.padding)
	sidebar:SetLayer(10)
	if(settings and settings.sidebar and settings.sidebar.width) then
		sidebar:SetWidth(settings.sidebar.width)
	else
		sidebar:SetWidth(150)
	end
	
	local resizeGripper = createResizeGripper(content)
	resizeGripper:SetPoint("CENTERY", content, "CENTERY")
	resizeGripper:SetPoint("CENTERX", sidebar, "RIGHT", Const.padding / 2, nil)

	modules = ModuleFrame(self)
	modules:SetPoint("TOPLEFT", sidebar, "BOTTOMLEFT")
	modules:SetPoint("TOPRIGHT", sidebar, "BOTTOMRIGHT")
	modules:SetPoint("BOTTOM", content, "BOTTOM", nil, -Const.padding)
	modules:SetLayer(9)

	bookmarks = BookmarksFrame(self)
	bookmarks:SetAllPoints(modules)
	bookmarks:SetVisible(false)
	bookmarks:SetLayer(9)

	local container = LibWiki.CreatePageContainer(content, "smoothstep", Const.animationDuration)
	container:SetPoint("TOPRIGHT", content, "TOPRIGHT", Const.padding, Const.padding)
	container:SetPoint("BOTTOMLEFT", modules, "BOTTOMRIGHT", Const.padding, 0)
	container:SetLayer(10)

	local forward = UI.CreateFrame("Texture", "", self)
	forward:SetPoint("TOPRIGHT", container:GetPageContentFrame(), "TOPRIGHT", 0, 0)
	forward:SetTexture("Rift", "btn_arrow_R_(disable).png.dds")
	forward:EventAttach(Event.UI.Input.Mouse.Left.Click, function() navigator:GoForward() end, "")
	forward:SetLayer(container:GetLayer() + 1)
	
	local back = UI.CreateFrame("Texture", "", self)
	back:SetPoint("TOPRIGHT", forward, "TOPLEFT", -2, 0)
	back:SetTexture("Rift", "btn_arrow_L_(disable).png.dds")
	back:EventAttach(Event.UI.Input.Mouse.Left.Click, function() navigator:GoBackward() end, "")
	
	local favorite = UI.CreateFrame("Texture", "", self)
	favorite:SetPoint("CENTERRIGHT", back, "CENTERLEFT")
	favorite:SetTexture("Rift", "icon_menu_achievements_(disabled).png.dds")
	favorite:SetHeight(23)
	favorite:SetWidth(23)
	favorite:EventAttach(Event.UI.Input.Mouse.Left.Click, function()
		if(bookmarks:IsBookmarked(pageModule, pagePath)) then
			bookmarks:Remove(pageModule, pagePath)
			favorite:SetTexture("Rift", "icon_menu_achievements_(disabled).png.dds")
		else
			bookmarks:Add(pageModule, pagePath, pageTitle)
			favorite:SetTexture("Rift", "icon_menu_achievements.png.dds")
		end
	end, "")
	
	forward:SetLayer(container:GetLayer() + 1)
	
	navigator = LibWiki.CreateNavigator(true, container)
	
	Command.Event.Attach(Event.LibWiki.Navigator.PageChanged, function(h, nav, module, path, title)
		if(nav == navigator) then
			pageModule, pagePath, pageTitle = module, path, title

			forward:SetTexture("Rift", navigator:GetPageHistoryPosition() == navigator:GetPageHistoryDepth() and "btn_arrow_R_(disable).png.dds" or "btn_arrow_R_(normal).png.dds")
			back:SetTexture("Rift", navigator:GetPageHistoryPosition() <= 1 and "btn_arrow_L_(disable).png.dds" or "btn_arrow_L_(normal).png.dds")
			favorite:SetTexture("Rift", bookmarks:IsBookmarked(module, path) and "icon_menu_achievements.png.dds" or "icon_menu_achievements_(disabled).png.dds")
			
			window:SetTitle(string.format("%s: %s", module, title))
		end
	end, "")

	PushPage("Rift", Const.indexPath)
end

-- Public methods
-- ============================================================================

function MainContext()
	return context
end

function MainWindow()
	if(type(window) == "function") then
		window()
	end
	return window
end

function PushPage(module, path)
	local w = MainWindow()
	w:FadeIn()
	navigator:PushPage(module, path)
end
